/*
Drawboard - Java applet used to make graphical teleconferences
Copyright (C) 2001  Tomek "TomasH" Zielinski, tomash@fidonet.org.pl

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package drawboard;

import java.io.*;
import java.net.*;
import java.util.*;

public class Server implements Runnable{
  volatile ServerSocket serverSocket = null;      // ServerSocket
  volatile int PORT = 7904;                       // default port number
  final short RIGHT_MARGIN=Main.RIGHT_MARGIN;     // Menu width
  final short BOTTOM_MARGIN=Main.BOTTOM_MARGIN;   // Colorpicker height
  volatile SingleClientHandle[] sch=null;         // table of client sockets
  int MAX=15;                                     // deault maximum
  volatile static int CURRENT=0;                  // temporary people counter
  int DELAY=30;                                   // delay between pings
  short DIM_X=600, DIM_Y=450;                     // image size
  int color=0xFFA6A8F8;
  int matrix[];
  int matrixtosend[];
  boolean runQuiet=false;

  // ===========================================================================
  // "Starter"
  public static void main(String[] args) throws IOException {
    new Server(args);
  }

  public static String hour(){
  Calendar now = Calendar.getInstance();
  return
    ((now.get(Calendar.HOUR)<10)?"0"+Integer.toString(now.get(Calendar.HOUR)):Integer.toString(now.get(Calendar.HOUR)))
    +":"+
    ((now.get(Calendar.MINUTE)<10)?"0"+Integer.toString(now.get(Calendar.HOUR)):Integer.toString(now.get(Calendar.MINUTE)))
    +":"+
    ((now.get(Calendar.SECOND)<10)?"0"+Integer.toString(now.get(Calendar.HOUR)):Integer.toString(now.get(Calendar.SECOND)));
  }

  // ===========================================================================
  // Constructor (with options reading)
  Server(String[] args){

    for (int a=0; a<args.length; a++)
      if (args[a].equalsIgnoreCase("-quiet")) runQuiet=true;
    if (!runQuiet){
      System.out.println("");
      System.out.println("Drawboard,  Copyright (C) 2001   Tomek \"TomasH\" Zielinski,  tomash@fidonet.org.pl");
      System.out.println("Drawboard comes with  ABSOLUTELY NO WARRANTY.  This  is  free software, and you are");
      System.out.println("welcome to redistribute it under the terms of the GNU Lesser General Public License");
      System.out.println("Use -help switch to get help");
      System.out.println();
    }
    for (int a=0; a<args.length; a++)
      if (args[a].equalsIgnoreCase("-h") || args[a].equalsIgnoreCase("--help") || args[a].equalsIgnoreCase("-help")){
        System.out.println("Command line switches:");
        System.out.println(" -x SIZEX       - applet horizontal size [pixels]     (default: 600)");
        System.out.println(" -y SIZEY       - applet vertical size [pixels]       (default: 450)");
        System.out.println(" -p PORT        - server port number                  (default: 7904)");
        System.out.println(" -m MAXCLIENTS  - maximum client number               (default: 15)");
        System.out.println(" -d DELAY       - delay between client ping [seconds] (default: 30)");
        System.out.println(" -c RRGGBB      - initial background color [hex]      (default: 0000FF)");
        System.out.println(" -quiet         - runs quiet (no text output)");
        System.out.println(" -help          - shows this help");
        System.out.println("");
        System.exit(0);
      }

    for (int a=0; a<args.length; a++){
      try{
          if (args[a].equalsIgnoreCase("-p")) PORT=java.lang.Integer.parseInt(args[a+1]);
          if (args[a].equalsIgnoreCase("-m")) MAX=java.lang.Integer.parseInt(args[a+1]);
          if (args[a].equalsIgnoreCase("-d")) DELAY=java.lang.Integer.parseInt(args[a+1]);
          if (args[a].equalsIgnoreCase("-x")) DIM_X=(short)java.lang.Integer.parseInt(args[a+1]);
          if (args[a].equalsIgnoreCase("-y")) DIM_Y=(short)java.lang.Integer.parseInt(args[a+1]);
          if (args[a].equalsIgnoreCase("-c")) color=
            255<<24+
            (java.lang.Integer.parseInt(args[a+1].substring(0,2),16))<<16+
            (java.lang.Integer.parseInt(args[a+1].substring(2,4),16))<<8+
            (java.lang.Integer.parseInt(args[a+1].substring(4,6),16));
      }catch(NumberFormatException e){
        System.err.println("Parameter '"+args[a]+" "+args[a+1]+"' is invalid, exiting.");
        System.exit(-1);
      }
      catch(ArrayIndexOutOfBoundsException e){
        System.err.println("Something is missing after "+args[a]+" switch, run with -help option to get help.");
        System.exit(-1);
      }

    }
    CURRENT=0;
    if (!runQuiet){
      System.out.println();
      System.out.println("Runtime options:");
      System.out.println("  applet dimensions: "+DIM_X+"x"+DIM_Y);
      System.out.println("  server port: "+PORT);
      System.out.println("  maximum client number: "+MAX);
      System.out.println("  delay between ping: "+DELAY);
      System.out.println("  initial background color:"+
        " R"+((color>>16)&0xFF)+
        " G"+((color>>8)&0xFF)+
        " B"+((color)&0xFF)
      );
    }
    try{
      DIM_X-=RIGHT_MARGIN;
      DIM_Y-=BOTTOM_MARGIN;
      matrix=new int[DIM_X*DIM_Y];
      for (int i=0; i<matrix.length; i++)matrix[i]=color;

      sch = new SingleClientHandle[MAX];
      serverSocket = new ServerSocket(PORT);
      Thread thread=new Thread(this);
      thread.start();
      while(true) addClient(serverSocket.accept());
    }catch(IOException e){if(!runQuiet)System.out.println(e);}
  }

  // ===========================================================================
  // Check current client number
  synchronized void setCurrent(){
    CURRENT=0;
    for (int c=0; c<MAX; c++)
      if (sch[c]!=null) CURRENT++;
//    if(!runQuiet)System.out.println(CURRENT+" users counted");
  }

  // ===========================================================================
  // Cyclic ping sending and connection checking
  synchronized void checkConnections(){
    setCurrent();
    for (int c=0; c<MAX; c++){
      if ((sch[c]!=null) && sch[c].finish==true){
        if (!runQuiet) System.out.println("** Number "+c+" hangs, disconnecting!");
        try{
          CURRENT--;
          sch[c].s.close();
          sch[c]=null;
        }catch(SocketException e){if (!runQuiet)System.out.println("**Socket error while closing socket "+e);}
        catch(IOException e){if (!runQuiet)System.out.println("**IO error while closing socket "+e);}
      } else if(sch[c]!=null){
        Pinger a=new Pinger((short)CURRENT);
//        if (!runQuiet)System.out.println("** Pinging "+c+" with value "+CURRENT);
        sch[c].addObject(a);
      }
    }
  }

  // ===========================================================================
  // Subthread calling checkConnections
  public void run(){
    while (true){
      try{Thread.sleep(DELAY*1000);}catch(InterruptedException e){if (!runQuiet)System.out.println("**Hey, I'm sleeping! "+e);}
      checkConnections();
    }
  }

  // ===========================================================================
  // Add (or refuse) new client
  void addClient(Socket socket){
    int c=0;
    try{while (
      sch[c]!=null) c++;}
    catch(ArrayIndexOutOfBoundsException e){
      try{
        socket.close();}
      catch(IOException ee){if (!runQuiet)System.out.println("Something wrong: "+ee);}
      if (!runQuiet)System.out.println("Refuse connection");
      return;
    }
    sch[c]=new SingleClientHandle(c,socket,this,runQuiet);
    if(!runQuiet) System.out.println(Server.hour()+"["+c+"] Connected from "+socket.getInetAddress().getHostAddress());

    setCurrent();
    sendToAll(new Pinger((short)CURRENT));
  }

  // ===========================================================================
  // Send object to all connected clients
  synchronized void sendToAll(Object o){
    if (o instanceof Reset){
      int color=((Reset)o).RGB;
      for (int i=0; i<matrix.length; i++)matrix[i]=color;
    }else{
      ((Shape)o).draw(matrix,(short)DIM_X);
    }
//    if (!runQuiet) System.out.println("Distributing to all clients object "+o);
    for (int c=0; c<MAX;c++){
      if (sch[c]!=null) sch[c].addObject(o);
    }
  }
}


// ===========================================================================
// ===========================================================================
class SingleClientHandle{
  volatile Vector v=null;     // Objects to send
  volatile int id;            // ID of this client
  volatile Socket s=null;     // socket
  volatile ObjectOutputStream oos=null;
  volatile ObjectInputStream ois=null;

  Server serverlink=null;
  boolean finish=false;
  boolean runQuiet;

  // ===========================================================================
  // Constructor
  SingleClientHandle(int i, Socket socket, Server server,   boolean r){
    v=new Vector();
    runQuiet=r;
    id=i;
    s=socket;
    serverlink=server;
//    System.out.println(id+" Initializing handle");
    try{
      oos=new ObjectOutputStream(s.getOutputStream());
      ois=new ObjectInputStream(s.getInputStream());
    }catch(IOException e){
      if (!runQuiet) System.out.println(Server.hour()+"["+id+"] Cannot get OO and OI streams, exiting. "+e);
      return;
    }
    SingleSender sender=new SingleSender(oos,id, this,runQuiet);
    sender.start();
    SingleListener listener=new SingleListener(ois,id, this,runQuiet);
    listener.start();
  }

  // ===========================================================================
  // Add to send queue
  synchronized void addObject(Object o){
//    if(!runQuiet) System.out.println(id+" Adding to queue: "+o);
    v.addElement(o);
  }

}

// ===========================================================================
// ===========================================================================
class SingleSender extends Thread{
  int id;
  boolean runQuiet;
  ObjectOutputStream oos=null;
  SingleClientHandle father=null;

  // ===========================================================================
  // Constructor
  SingleSender(ObjectOutputStream ooooss, int i, SingleClientHandle shc, boolean r){
    id=i;
    runQuiet=r;
    oos=ooooss;
    father=shc;
  }

  // ===========================================================================
  // Sending responsible subthread
  public void run(){
    try{
      if (!runQuiet) System.out.println(Server.hour()+"["+id+"] Sender: Sending archive packet");
      oos.writeObject(new Pinger((short)Server.CURRENT));
      oos.writeObject(new Archive(father.serverlink.matrix,father.serverlink.DIM_X, father.serverlink.DIM_Y));
      oos.flush();
    }catch(IOException e){
      if (!runQuiet) System.out.println(Server.hour()+"["+id+"] Sender: Archive data sending error! "+e);
      father.finish=true;
    }

    while (!father.finish){
      while(father.v.isEmpty() && !father.finish){
        try{Thread.sleep(500);}catch(InterruptedException e){if (!runQuiet) System.out.println(Server.hour()+"["+id+"] Sender: Hey, I'm sleeping! "+e);}
      }
      if (!father.v.isEmpty()){
        try{
          Object a=father.v.firstElement();
//          if (!runQuiet) System.out.println(id+" Sender: sending "+a);
          father.v.removeElementAt(0);
          oos.writeObject(a);
          oos.flush();
        }catch(IOException e){
          if (!runQuiet) System.out.println(Server.hour()+"["+id+"] Sender: Could not send anything "+e);
          father.finish=true;
        }
      }
    }
    try{
      oos.close();
    }catch(IOException e){
      if (!runQuiet) System.out.println("IOException while closing OOS, ignoring "+e );
    }
    if (!runQuiet) System.out.println(Server.hour()+"["+id+"] Sender: Subthread ending");
  }
}


// ===========================================================================
// ===========================================================================
class SingleListener extends Thread{
  int id;
  boolean runQuiet;
  ObjectInputStream ois=null;
  SingleClientHandle father=null;

  // ===========================================================================
  // Constructor
  SingleListener(ObjectInputStream ooiiss, int i, SingleClientHandle shc,  boolean r){
    ois=ooiiss;
    runQuiet=r;
    id=i;
    father=shc;
  }


  // ===========================================================================
  // Listen responsible thread
  public void run(){

    while (!father.finish){
      try{
        Object o=ois.readObject();
        if (!runQuiet) System.out.println(Server.hour()+"["+id+"] Received object "+o.getClass().getName()+" (size: "+Communicator.getSize(o)+" bytes)");
        father.serverlink.sendToAll(o);
      }catch(IOException e){if (!runQuiet) System.out.println(Server.hour()+
        "["+id+"] Listener: While reading got: "+e);
        father.finish=true;
      }
      catch(ClassNotFoundException e){
        if (!runQuiet) System.out.println(Server.hour()+"["+id+"] Listener: Strange class, trying to ignore: "+e);
      }
    }
    try{
      ois.close();
    }catch(IOException e){
      if (!runQuiet) System.out.println("IOException while closing OIS, ignoring "+e );
    }
    if (!runQuiet) System.out.println(Server.hour()+"["+id+"] Listener: ending");
  }
}

